package App::JESP::Cmd::Command;
$App::JESP::Cmd::Command::VERSION = '0.014';
use App::Cmd::Setup -command;
use strict; use warnings;

=head1 NAME

App::JESP::Cmd::Command - Super class for all CLI commands.

=cut

1;
